#!/usr/bin/python2
#
import execproxy
import os,stat
import unittest

execproxy.remote_program=os.path.join(os.getcwd(),'src','remote.py')

class newdirTestCase(unittest.TestCase):
    def runTest(self):
	ep=execproxy.exec_proxy('localhost')
	assert ep
	f=ep.newdir()
	self.failIf(f==None,'newdir() call failed')

	self.failIf(not os.path.isdir(f),'reported created dir does not exist')

class unitTestCases(unittest.TestCase):
    def setUp(self):
	self.ep=execproxy.exec_proxy('localhost')
	assert self.ep
	self.newdir=self.ep.newdir()
	assert self.newdir!=None
	if not os.path.isdir(self.newdir):
	    raise 'directory '+repr(self.newdir)+' not created'

    def doChdir(self):
	x=self.ep.chdir(self.newdir)
	self.assertEqual(x,0,'chdir to newdir failed')

    def test_chdir(self):
	self.doChdir()

    def test_mkdir(self):
	self.doChdir()
	x=self.ep.mkdir('xyz')
	self.assertEqual(x,0,'mkdir of xyz returned error')
	self.failIf(not os.path.isdir(self.newdir+'/xyz'),'directory xyz is not created')

    def test_getcwd(self):
	self.doChdir()
	p=self.ep.getcwd()
	self.assertEqual(p,self.newdir,'getcwd returned weird result')

    def test_rmdir(self):
	self.test_mkdir()
	x=self.ep.rmdir(self.newdir+'/xyz')
	self.assertEqual(x,0,'rmdir of xyz returned error')
	self.failIf(os.path.exists(self.newdir+'/xyz'),'directory xyz not removed')

	x=self.ep.rmdir(self.newdir+'/non-existant-dir')
	self.assertEqual(x,1,'rmdir of non-existant-dir failed')

    def test_isdir(self):
	t=self.ep.isdir(self.newdir)
	self.assertEqual(t,1,'isdir of my-dir failed')

	t=self.ep.isdir('/non/existant/directory')
	self.assertEqual(t,0,'isdir of /non/existant/directory-dir failed')

	self.doWrite('x','myfile')
	t=self.ep.isdir('myfile')
	self.assertEqual(t,0,'isdir of myfile failed')

    def doWrite(self,data,name):
	self.doChdir()
	x=self.ep.write(data,name)
	self.assertEqual(x,0,'write of '+repr(name)+' failed')

    def test_write(self):
	self.doWrite('s;doi\n','myfile')
	self.failIf(not os.path.exists(self.newdir+'/myfile'),'myfile is not created')

	data=open(self.newdir+'/myfile').read()
	self.assertEqual(data,'s;doi\n','contents of written file incorrect')

    def test_isfile(self):
	self.doWrite('','myfile')
	x=self.ep.isfile('myfile')
	self.assertEqual(x,1,'isfile of myfile failed')

	# isfile on something non existant should return 0
	x=self.ep.isfile('non-existant-file')
	self.assertEqual(x,0,'isfile of non-existant-file failed')

	# isfile on a dir should return 0
	x=self.ep.isfile(self.newdir)
	self.assertEqual(x,0,'isfile of newdir failed')

    def test_listdir(self):
	x=self.ep.listdir('/usr/lib')
	assert x!=None
	self.failIf(len(x)!=2,'returned tuple of listdir !=2')
	ydir=os.listdir('/usr/lib')
	for y in ydir:
	    if os.path.isdir('/usr/lib/'+y):
		self.failIf(y not in x[0],'Missing dir /usr/lib/'+y)
		x[0].remove(y)
	    elif os.path.isfile('/usr/lib/'+y):
		self.failIf(y not in x[1],'Missing file /usr/lib/'+y)
		x[1].remove(y)
	    else:
		self.fail('Weird /usr/lib/'+y+' thingie')
	self.failIf(x[0]!=[],'Too many dirs returned')
	self.failIf(x[1]!=[],'Too many files returned')
	
    def test_exists(self):
	t=self.ep.exists(self.newdir)
	self.failIf(t!=1,'Cannot find newdir')

	t=self.ep.exists('/non-existant/dir/something/bla')
	self.failIf(t!=0,'Can find something non-existant')

	self.doWrite('','exist-file')
	t=self.ep.exists('exist-file')
	self.failIf(t!=1,'Cannot find newdir/exist-file')

    def test_chmod(self):
	self.doWrite('blup','myfile')
	path=self.newdir+'/myfile'
	s=os.stat(path)

#    def doWrite(self,data,name):

    def test_symlink(self):
	self.doChdir()
	t=self.ep.symlink('file','link')
	self.failIf(t!=0,'symlink creation failed')
	s=os.lstat(self.newdir+'/link')
	self.failIf(not stat.S_ISLNK(s[stat.ST_MODE]),'Symlink does not exist')

    def test_fifo(self):
	self.doChdir()
	t=self.ep.mkfifo('fifo')
	self.failIf(t!=0,'fifo creation failed')
	s=os.stat(self.newdir+'/fifo')
	self.failIf(not stat.S_ISFIFO(s[stat.ST_MODE]),'Fifo does not exist')
	self.failIf(stat.S_IMODE(s[stat.ST_MODE])!=0600,'Access rights bad')

    def test_stat(self):
	t=self.ep.stat(self.newdir)
	self.failIf(t==None,'Could not stat newdir')
	s=os.stat(self.newdir)
	self.failIf(t!=s,'Stat data is not equal')

def suite():
    s=unittest.TestSuite()
    s.addTest(newdirTestCase())
    s2=unittest.makeSuite(unitTestCases,'test_')
    s.addTest(s2)
    return s

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite())
