#!/usr/bin/python2
# $Id: __init__.py,v 1.1 2003/10/20 07:57:00 hat Exp $
#
# exec_proxy: Local proxy class for executing commands at a remote machine
#
import os,popen2,pickle

connection_program='ssh -x'
remote_program='\'python2 -c "from exec_proxy.remote import run ; run()"\''

class _Proxy_IO:
    """Execution class of the local proxy. sets up and maintains the
       connection to the remote_proxy object. Also maintains the error
       state."""

    def __init__(self,host,host_is_cmd=0):
	global connection_program, remote_program

	if ' ' in host or host_is_cmd:
	    cmd=host
	else:
	    cmd=connection_program+' '+host+' '+remote_program

	(fpi,fpo)=popen2.popen2(cmd,0)
	self.fpin=pickle.Unpickler(fpi)
	self.fpout=pickle.Pickler(fpo)

	self.error_state=0	# We are not in error state
	self.error_string=''	# empty human-readable error string

    # Public interface (i.e. the advertised part)
    def is_ok(self):
	return not self.error_state

    def get_error(self):
	return self.error_string

    def reset_error(self):
	self.error_state=0
	self.error_string=''

    # non-public interface (i.e. non-advertised part)
    def set_error(self,err_string):
	if err_string=='':
	    err_string='Unknown error'	# we want a non-empty default error
	self.error_state=1
	self.error_string=err_string

    def execute(self,cmd,parms):
	"""Send the tuple (cmd,parms) to the remote end, and wait for
	   an answer. Either the answer is (0,ret-value) which means
	   'ok', or the answer is (1,'error-string') which means 'error
	   found while processing'.
	   Returns either (0,ret-value) if execution was ok, or (1,None)
	   otherwise."""

	if not self.is_ok():	# we are in error state, so all cmds are
	    return (1,None)	# considered to be an error

	try:
	    self.fpout.dump( (cmd,parms) )
	except:
	   self.set_error('Error while writing to remote host')
	   return (1,None)

	try:
	    (status,ret)=self.fpin.load()
	except:
	    self.set_error('Error while reading from remote host')
	    return (1,None)

        #print "local: pio: read="+repr((status,ret))
	if status==0:
	    return (status,ret)
	self.set_error(ret)	# an error happened. ret describes the error
	return (1,None)	# and return an error to the caller

#
# # # # # # # # # # # # # # # # # # # # # # # # #
#

class Exec_proxy:
    """Proxy class that runs commands at a remote machine"""

    def __init__(self,host, host_is_cmd=0):
	self._pio=_Proxy_IO(host, host_is_cmd)

    # proxy functions
    #
    def newdir(self):	return self._exec('newdir',(),'')
    def mkdir(self,new_dir):
			return self._exec('mkdir',(new_dir,),-1)
    def chdir(self,dir_path):
			return self._exec('chdir',(dir_path,),-1)
    def rmdir(self,dir_path):
			return self._exec('rmdir',(dir_path,),-1)
    def getcwd(self):	return self._exec('getcwd',(),'')

    def isdir(self,path):
			return self._exec('isdir',(path,),-1)
    def isfile(self,path):
			return self._exec('isfile',(path,),-1)
    def exists(self,path):
			return self._exec('exists',(path,),-1)

    def listdir(self,path):
			return self._exec('listdir',(path,),None)
    def chmod(self,path,mode=0600):
		        return self._exec('chmod',(path,mode,),-1)
    def stat(self,file_path):
			return self._exec('stat',(file_path,),None)
    def mkfifo(self,file_path,mode=0600):
			return self._exec('mkfifo',(file_path,mode),-1)
    def symlink(self,old_file,new_file):
			return self._exec('symlink',(old_file,new_file),-1)
    def unlink(self,file_path):
			return self._exec('unlink',(file_path,),-1)

    def upload(self,local_file,file_path,overwrite=0):
			if not self.is_ok(): return -1
			data=self._load(local_file)
			if data==None: return -1
			return self._exec('write',(data,file_path,overwrite),-1)
    def download(self,file_path,local_file,overwrite=0):
			data=self._exec('read',(file_path,),None)
			if data==None: return -1
			if self._save(data,local_file,overwrite):
			    return -1
			return 0
    def write(self,data,file_path,overwrite=0):
			return self._exec('write',(data,file_path,overwrite),-1)
    def read(self,file_path):
			return self._exec('read',(file_path,),None)

    def run(self,cmd,input=''):
			return self._exec('run',(cmd,input),None)

    # error functions (forwarded to self._pio object)
    #
    def is_ok(self):	    return self._pio.is_ok()
    def get_error(self):    return self._pio.get_error()
    def reset_error(self):  return self._pio.reset_error()


    # Internal functions
    #
    def _exec(self,cmd,parms,error_return):
	"""Run cmd with parms at the remote end by using the self._pio
	   object. Returns the ret-value of the remote end if execution
	   was ok, or the supplied error_return in case of failure."""
	(status,ret)=self._pio.execute(cmd,parms)
	if status==0:
	    return ret
	return error_return

    def _load(self,local_file):
	"""Load local file local_file, and return its contents."""
	try:
	    fp=open(local_file,'rb')
	    data=fp.read()
	    fp.close()
	    return data
	except:
	    return None
    def _save(self,data,local_file,overwrite):
	"""Save data to local file local_file. Do not overwrite is overwrite==0
	   Return 0 if ok,, 1 otherwise"""
	if data==None: return 1
	if overwrite or not os.path.exists(local_file):
	    try:
		fp=open(local_file,'wb')
		fp.write(data)
		fp.close()
		return 0    # ok
	    except:
		return 1    # error
	return 1    # overwrite==0 and file exists -> error
