#!/usr/bin/python2
# $Id: remote.py,v 1.1 2003/10/20 07:57:00 hat Exp $
#
# Remote program started by exec_proxy class (in execproxy.py) that
# performs commands on behalf of the former class.
#
import sys,pickle,os,tempfile,popen2,select

class RemoteError:	# used for internal exceptions
    pass

class Remote:
    def __init__(self,fpi,fpo):
	self.fpi=fpi
	self.fpo=fpo
	self.fpin=pickle.Unpickler(fpi)
	self.fpout=pickle.Pickler(fpo)

	self.dict={ 'newdir': (0,self.__newdir), 'mkdir':  (1,self.__mkdir),
		    'chdir':  (1,self.__chdir),  'rmdir':  (1,self.__rmdir),
		    'getcwd': (0,self.__getcwd), 'isdir':  (1,self.__isdir),
		    'isfile': (1,self.__isfile), 'exists': (1,self.__exists),
		    'listdir':(1,self.__listdir),'chmod':  (1,self.__chmod),
		    'chmod':  (2,self.__chmod),  'stat':   (1,self.__stat),
		    'mkfifo': (2,self.__mkfifo), 'symlink':(2,self.__symlink),
		    'unlink': (1,self.__unlink), 'write':  (3,self.__write),
		    'read':   (1,self.__read),   'run':    (2,self.__run)
		}

    def run(self):
	self.quit=0

	while not self.quit:
	    data=self._read_network()
	    if data==None:
		self.quit=1
	    else:
		#sys.stderr.write("remote: received "+repr(data)+'\n')
		(res,data)=self._process_cmd(data[0],data[1])
		#sys.stderr.write("remote: result="+repr((res,data))+'\n')
		if self._write_network( (res,data) ):
		    self.quit=1

    def _read_network(self):
	"""Read a tuple (cmd,parms) from the network using the unpickler.
	   Return the tuple, or None in case of failure"""
	try:
	    (cmd,parms)=self.fpin.load()
	    return (cmd,parms)
	except:
	    return None

    def _write_network(self,data):
	"""Write the data onto the network using the pickler. Return 0 if
	   ok, 1 if error"""
	try:
	    self.fpout.dump(data)
	    self.fpo.flush()
	    return 0
	except:
	    return 1

    def _process_cmd(self,cmd,parms):
	"""Try to process the command with its parameters. Return (0,result)
	   or (1,error-string)."""
	m=self.dict.get(cmd)
	if m==None:
	    return (1,'Cannot handle command '+repr(cmd))
	else:
	    if len(parms)!=m[0]:
		return (1,'Number of parameters of '+repr(cmd)+' incorrect')
	    try:
		ret=m[1](parms)
		return (0,ret)
	    except OSError, err:
		return (1,err)
	    except IOError, err:
		return (1,err)
	    except RemoteError, err:
		return (1,err)
	    except:
		return (1,cmd+repr(parms)+' failed')


    def __newdir(self,parms):
	fn=tempfile.mktemp()
	os.mkdir(fn,0700)
	return fn
    def __mkdir(self,parms):
	os.mkdir(parms[0])
	return 0
    def __chdir(self,parms):
	os.chdir(parms[0])
	return 0
    def __rmdir(self,parms):
	os.rmdir(parms[0])
	return 0
    def __getcwd(self,parms):	return os.getcwd()
    def __isdir(self,parms):	return os.path.isdir(parms[0])
    def __isfile(self,parms):	return os.path.isfile(parms[0])
    def __exists(self,parms):	return os.path.exists(parms[0])
    def __listdir(self,parms):
	dl,fl=[],[]
	for n in os.listdir(parms[0]):
	    if os.path.isdir(parms[0]+os.sep+n):
		dl.append(n)
	    else:
		fl.append(n)
	return (dl,fl)
    def __chmod(self,parms):
	os.chmod(parms[0],parms[1])
	return 0
    def __stat(self,parms):	return os.stat(parms[0])
    def __mkfifo(self,parms):
	os.mkfifo(parms[0],parms[1])
	return 0
    def __symlink(self,parms):
	os.symlink(parms[0],parms[1])
	return 0
    def __unlink(self,parms):
	os.unlink(parms[0])
	return 0
    def __write(self,parms):
	(data,filename,overwrite)=parms
	if overwrite or not os.path.exists(filename):
	    fp=open(filename,'wb')
	    fp.write(data)
	    fp.close()
	    return 0
	raise RemoteError,'Cannot overwrite: file eixsts'
    def __read(self,parms):
	fp=open(parms[0],'rb')
	data=fp.read()
	fp.close()
	return data
    def __run(self,parms):
	(cmd,input)=parms
	p3=popen2.Popen3(cmd,1)
	output,errors='',''
	if len(input)==0: p3.tochild.close()
	outputeof,erroreof=0,0  # not seen stdout_eof and stderr_eof
	while 1:
	    inl=[]	# child input (and thus our output!)
	    if len(input)>0: inl.append(p3.tochild)
	    outl=[] # child output (and thus our input!)
	    if not outputeof: outl.append(p3.fromchild)
	    if not erroreof:  outl.append(p3.childerr)
	    (outl,inl,errl)=select.select(outl,inl,[])
	    if len(inl)>0:
		p3.tochild.write(input[0])
		input=input[1:]
		if len(inl)==0: p3.tochild.close()
	    if not outputeof and p3.fromchild in outl:
		n=p3.fromchild.read(1)
		output=output+n
		if len(n)<1:
		    outputeof=1
		    p3.fromchild.close()
	    if not erroreof and p3.childerr in outl:
		n=p3.childerr.read(1)
		errors=errors+n
		if len(n)<1:
		    erroreof=1
		    p3.childerr.close()
	    if outputeof and erroreof:
		break
	if len(input)!=0: p3.tochild.close()
	exitcode=p3.wait()
	return (exitcode,output,errors)

def run():
    global Remote
    r=Remote(sys.stdin,sys.stdout)
    r.run()
    sys.exit(0)

if __name__ == '__main__':
    run()
    sys.exit(0)
